# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# Controlador encargado del resumen inicial del usuario o el proyecto. Este controlador es utilizado desde las secciones:
# * Sección inicio: visualizar los datos relativos al usuario
# * Sección proyectos: visualizar y los datos relativos al proyecto

class InfoController  < ApplicationController

  before_filter :obtiene_proyectos
  before_filter :comprobar_tasas_cambio

  # El metodo comprobar_periodo_identificador_financiador se encuentra en el ApplicationController
  before_filter :comprobar_periodo_identificador_financiador

  def index
    if @proyecto
      c_periodos = "tipo_periodo.oficial = 1 AND tipo_periodo.grupo_tipo_periodo <> 'prorroga' AND periodo_cerrado = 0 "
      @periodos = @proyecto.periodo.find(:all, :include => :tipo_periodo, :conditions => c_periodos, :order => ("fecha_inicio"))
      c_periodos_internos = "tipo_periodo.oficial = 0 AND tipo_periodo.grupo_tipo_periodo <> 'prorroga' AND periodo_cerrado = 0"
      @periodos_internos = @proyecto.periodo.find(:all, :include => :tipo_periodo, :conditions => c_periodos_internos, :order => ("fecha_inicio"))
      c_prorrogas = "tipo_periodo.grupo_tipo_periodo = 'prorroga' AND periodo_cerrado = 0"
      @prorrogas = @proyecto.periodo.find(:all, :include => :tipo_periodo, :conditions => c_prorrogas, :order => ("fecha_inicio"))
      condiciones_tareas = ["estado_tarea.activo IS TRUE AND proyecto_id = ? AND usuario_asignado_id = ?",@proyecto.id, @usuario_identificado.id]
    elsif @agente
      condiciones_tareas = ["estado_tarea.activo IS TRUE AND agente_id = ? AND usuario_asignado_id = ?",@agente.id, @usuario_identificado.id]
    else
      condiciones_periodo = {}
      proyectos = @usuario_identificado.proyecto.select {|p| p.estado_actual && !p.estado_actual.definicion_estado.cerrado }  
      c_periodos = ["tipo_periodo.oficial = 1 AND tipo_periodo.grupo_tipo_periodo <> 'prorroga' AND periodo_cerrado = 0 AND proyecto_id IN (:proyecto_id)", {:proyecto_id => proyectos}]
      @periodos = Periodo.find(:all, :order => ("fecha_inicio"), :include => :tipo_periodo, :conditions => c_periodos) 
      c_periodos_internos = ["tipo_periodo.oficial = 0 AND tipo_periodo.grupo_tipo_periodo <> 'prorroga' AND periodo_cerrado = 0 AND proyecto_id IN (:proyecto_id)", {:proyecto_id => proyectos}]
      @periodos_internos =  Periodo.find(:all, :order => ("fecha_inicio"), :include => :tipo_periodo, :conditions => c_periodos_internos) 
      c_prorrogas = ["tipo_periodo.grupo_tipo_periodo = 'prorroga' AND periodo_cerrado = 0 AND proyecto_id IN (:proyecto_id)", {:proyecto_id => proyectos}]
      @prorrogas = Periodo.find(:all, :order => ("fecha_inicio"), :include => :tipo_periodo, :conditions => c_prorrogas) 
      condiciones_tareas = ["estado_tarea.activo IS TRUE AND proyecto_id IN (?) AND usuario_asignado_id = ?",@usuario_identificado.proyecto, @usuario_identificado.id]
      c_posibles_prorrogas = ["fecha_inicio_aviso_peticion_prorroga IS NOT NULL AND fecha_limite_peticion_prorroga IS NOT NULL AND fecha_inicio_aviso_peticion_prorroga < ? AND fecha_limite_peticion_prorroga > ?", Date.today, Date.today]
      @proyectos_fechas_prorroga = @usuario_identificado.proyecto.where(c_posibles_prorrogas)
    end
    @tareas = Tarea.find( :all, :conditions => condiciones_tareas, :include => [:tipo_tarea, :estado_tarea], :order => ("fecha_inicio"))
    render 'proyecto' if @proyecto
    render 'agente' if @agente
    render 'usuario' unless @proyecto || @agente 
  end

  def mas_proyectos
    render :update do |page|
      page.replace params[:update], :partial => 'proyectos'
    end
  end

  # METODOS DE Plantillas de documentos asociados Ficha Resumen
  #++

  # Genera un documento  (ojo, el codigo hay que pasarlo a una libreria para unificar y simplificar controladores)
  def ficha_resumen
    @proyecto = Proyecto.find_by_id(params[:proyecto_id])
    @documento = Documento.includes("etiqueta").where("etiqueta.nombre" => "Ficha Resumen", "etiqueta.tipo" => "plantilla").first

    if @documento && File.exists?(@documento.adjunto.path)
      nom_fich = @proyecto.nombre + "." + @documento.adjunto_file_name
      fichero = Tempfile.new("proyecto_" + @proyecto.id.to_s + "_" + SecureRandom.hex)
      fichero.close

      valores = @proyecto.campos_plantilla.merge(campos_plantilla)

      begin
        source = Word::WordDocument.new(@documento.adjunto.path)
        # Hay que cambiar la forma de hacerlo para que se lean las claves del documento y se busque el valor
        valores.each { |k,v| source.replace_all("{{" + k.to_s.upcase + "}}", v.to_s) }
        source.save(fichero.path)
        send_file fichero.path, :filename => nom_fich, :type => @documento.adjunto_content_type, :disposition => 'inline'
      rescue => ex
        logger.error ex.message
        logger.error ex.backtrace
        msg_error = _("Se produjo un error leyendo la plantilla: %{msg_err}")%{:msg_err => ex.message}
        #redirect_to :action => 'listado'
      end
    else
      msg_error _("No se pudo encontrar una plantilla de 'ficha_resumen'.") + " " + _("Contacte con el administrador del sistema.")
      #redirect_to :action => 'listado'
    end
  end

 private

  def obtiene_proyectos
    @proyectos = @agente.proyecto_implementador.includes("definicion_estado").where("definicion_estado.aprobado" => true, "definicion_estado.cerrado" => false, "convenio_id" => nil) if @agente
    @proyectos = @usuario_identificado.proyecto.includes(:definicion_estado).where("NOT definicion_estado.cerrado").
                                       order("definicion_estado.orden desc, proyecto.nombre").
                                       paginate(:page => (params[:page] || 1), :per_page => 3) unless @proyecto || @agente
  end

  # Comprueba que existan tasas de cambio para presupuesto y para gastos
  def comprobar_tasas_cambio
    if @proyecto && @proyecto.estado_actual && @proyecto.definicion_estado.aprobado
      mensaje = ""
      # Comprueba que haya TC de gasto para cada moneda configurada
      tasas = TasaCambio.where(objeto: "Gasto").joins(:etapa).where("etapa.proyecto_id" => @proyecto.id)
      @proyecto.moneda.each do |moneda|
        if tasas.where(moneda_id: moneda.id).empty?
          mensaje += "<br>".html_safe unless mensaje.empty?
          mensaje += _("No se ha configurado una tasa de cambio para la moneda %{mon}.")%{mon: moneda.abreviatura}
        end
      end 
      msg_error mensaje, now: true unless mensaje.empty?
    end
  end
end
