# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
# estado

class Estado < ActiveRecord::Base
  #untranslate_all
  belongs_to :definicion_estado
  belongs_to :proyecto
  belongs_to :financiacion
  belongs_to :usuario

  # Comprobamos cambio de estado en proyectos
  validate :verificar_condiciones_estado
  validate :verificar_cierre_proyecto_tareas
  validate :verificar_documentacion
  after_save :verificar_cierre_proyecto

  private

	# Comprueba las condiciones de paso del estado
    def verificar_condiciones_estado
      # Si pasamos a un estado aprobado, pero no existe cuenta de subvencion... casca
      errors.add("proyecto", _("El proyecto no tiene definida cuenta de recepción de la Subvención.")) if (self.proyecto.nil? || self.proyecto.libro_principal.nil?) && self.estado_actual && self.definicion_estado.aprobado
      # Revisa las condiciones de cambio de los plugins (si las hubiera)
      Plugin.activos.each do |plugin|
        begin
          eval(plugin.clase + "::Estado").verificar_condiciones_estado self
        rescue => ex 
        end
      end
      return self.errors.empty?
    end

 	# Comprueba si es un cierre del proyecto y si tiene tareas activas
    def verificar_cierre_proyecto_tareas
      # OJO: Errores en los logs del tipo:
      # NoMethodError (undefined method `activo' for nil:NilClass):
      #   app/models/estado.rb:54:in `block in verificar_cierre_proyecto_tareas'
      #   app/models/estado.rb:54:in `verificar_cierre_proyecto_tareas'
      #   app/controllers/estado_controller.rb:69:in `modificar_estado'
      #
      if self.estado_actual and self.definicion_estado.cerrado and proyecto.tarea.find{|t| t.estado_tarea.nil? || t.estado_tarea.activo}
        errors.add("proyecto", _("No se puede cambiar el proyecto a un estado de tipo cerrado hasta haber cerrado todas las tareas del proyecto."))      
      end
    end
    # Comprobamos que todas la documentacion-etiquetada definida para pasar de estado esta en el proyecto
    def verificar_documentacion
      if self.estado_actual
        todas_etiquetas = Array.new
        for documento in proyecto.documento
          todas_etiquetas << documento.etiqueta
        end
        todas_etiquetas = todas_etiquetas.flatten
        unless (definicion_estado.etiqueta - todas_etiquetas).empty?
          mensaje = _("No se puede cambiar de estado. Compruebe que tiene los documentos necesarios y que están adecuadamente etiquetados.") + " "
          mensaje << _("Los documentos etiquetados necesarios para cambiar el estado actual son:") + "    "
          mensaje << definicion_estado.etiqueta.inject("") {|todo, a| todo + '  "' + a.nombre + '"   ' }
          errors.add("proyecto", mensaje)
        end
      end
    end


	# Comprueba si es un cierre del proyecto
    def verificar_cierre_proyecto
      # Si es el estado actual y es cerrado...
      if self.estado_actual && self.definicion_estado.cerrado
        # Para pacs, aplica remanentes sobre el pac siguiente
        self.proyecto.pac_siguiente.aplica_remanentes if self.proyecto.pac_siguiente
      end
    end

end
