# encoding: UTF-8
#--
#
#################################################################################
# Copyright 2010-2013 Free Software's Seed, CENATIC y IEPALA
#
# Licencia con arreglo a la EUPL, Versión 1.1 o –en cuanto sean aprobadas por la Comisión Europea– 
# versiones posteriores de la EUPL (la «Licencia»);
# Solo podrá usarse esta obra si se respeta la Licencia.
# Puede obtenerse una copia de la Licencia en:
#
# http://www.osor.eu/eupl/european-union-public-licence-eupl-v.1.1
#
# Salvo cuando lo exija la legislación aplicable o se acuerde por escrito, 
# el programa distribuido con arreglo a la Licencia se distribuye «TAL CUAL»,
# SIN GARANTÍAS NI CONDICIONES DE NINGÚN TIPO, ni expresas ni implícitas.
# Véase la Licencia en el idioma concreto que rige los permisos y limitaciones que establece la Licencia.
#################################################################################
#
#++
class Periodo < ActiveRecord::Base
  belongs_to :tipo_periodo
  belongs_to :proyecto
  has_many :tarea, :dependent => :destroy

  before_destroy :comprobar_estado_proyecto

  validates_presence_of :fecha_inicio, :message => _("Fecha inicio") + " " + _("no puede estar vacío.")
  validates_presence_of :fecha_fin, :message => _("Fecha fin") + " " + _("no puede estar vacío.")
  validates_presence_of :fecha_fin, :message => _("Fecha informes") + " " + _("no puede estar vacío.")
  

  # Esto es una chapuza que habria que eliminar (esta metido a capon el id del informe final)
  validates_uniqueness_of :tipo_periodo_id, :scope => :proyecto_id, :if => Proc.new{|periodo| periodo.tipo_periodo_id == 1}, :message => _("Informe Final repetido.")

  validate :comprobar_estado_proyecto 
  validate :comprobar_fechas
  validate :comprobar_fechas_etapa


  def estado_color
    color = ""
    hoy = Date.today
    if fecha_fin && fecha_inicio && fecha_informe
      if hoy > fecha_fin && hoy < fecha_informe
        color = "verde"
      elsif hoy > fecha_informe
        color = "rojo"
      end
    end unless periodo_cerrado
    return color
  end
  
  def estado
     periodo_cerrado ? _("Cerrado/Aceptado") : _("Abierto") 
  end
  
  def estado_cerrado
    estado = "cerrado"
    for tarea in self.tarea
      estado = "abierto" unless tarea.estado_tarea.nil? || tarea.estado_tarea.activo
    end
    return estado == "abierto" ? false : true
  end

  def estado_abierto
    abierto = tarea.empty? 
    tarea.each { |t| abierto = true unless t.estado_tarea && !t.estado_tarea.activo }
    return abierto
  end
  
  def porcentaje_tiempo
    hoy = Date.today
    if hoy < fecha_inicio 
      0
    elsif hoy > fecha_fin
      1
    else
      (hoy - fecha_inicio).to_f / (fecha_fin - fecha_inicio).to_f
    end
  end

  def porcentaje_tiempo_rotulo
    hoy = Date.today
    if hoy < fecha_inicio 
      _("Faltan %{num} días para comenzar") % {:num => (fecha_inicio - hoy).to_i}
    elsif  hoy > fecha_fin 
      _("%{num} días desde su finalización") % {:num => (hoy - fecha_fin).to_i}
    else 
      _("%{num} días restantes") % {:num => (fecha_fin - hoy).to_i}
    end
  end

  def tiempo_al_informe
    ((fecha_informe - Date.today).to_i.to_s + " dias") if fecha_informe
    
    # if hoy > fecha_informe
    #   _("%{num} días de retraso") % {:num => (hoy - fecha_informe).to_i}
    # elsif hoy > fecha_fin && hoy < fecha_informe
    #   _("%{num} días restantes") % {:num => (fecha_informe - hoy).to_i}
    # else
    #   _("Periodo en curso. Faltan %{num} días") % {:num => (fecha_fin - hoy).to_i}
    # end
  end

 private

  # Comprueba que las fechas del periodo sean coherentes entre si
  def comprobar_fechas
    # Primero comprobamos que existen fechas que comprobar. Si no hubiese fechas ya hay comrpobaciones que impiden guardar.
    if fecha_inicio && fecha_fin and fecha_informe
      if fecha_fin < fecha_inicio
        # Comprueba que la fecha de inicio sea anterior a la de fin
        errors.add("fecha_fin", _("Fecha fin tiene que ser mayor que fecha inicio"))
      elsif  fecha_informe < fecha_fin
        # Comprueba que la fecha de inicio sea anterior a la de fin
        errors.add("fecha_fin", _("La fecha del informe tiene que ser mayor que fecha fin"))
      else 
        errores = false
        # Comprueba que no se solapen fechas (una fecha de inicio de un periodo anterior a la de fin de otro)
        (proyecto.periodo.where(:tipo_periodo_id => self.tipo_periodo_id) - [self]).each do |p|
          errores = true unless (self.fecha_inicio < p.fecha_inicio && self.fecha_fin < p.fecha_inicio) || (self.fecha_inicio > p.fecha_fin && self.fecha_fin > p.fecha_fin)
        end
        errors[:base] << _("El periodo no puede solaparse con otro periodo del mismo tipo") if errores
      end
    end
    return errors.nil?
  end
  
  # Compruba la coherencia de las fechas respecto a las etapas del proyecto. SEgun el tipo de periodo
  def comprobar_fechas_etapa
    # Primero comprobamos que existen fechas que comprobar. Si no hubiese fechas ya hay comrpobaciones que impiden guardar.
    if fecha_inicio && fecha_fin
      if tipo_periodo.grupo_tipo_periodo == "seguimiento" && (proyecto.fecha_de_inicio > fecha_inicio || proyecto.fecha_de_fin < fecha_inicio)
        errors[:base] <<  _("Los periodos de seguimiento deben estar dentro de las etapas del proyecto") 
      end
    end
  end
  

  def comprobar_estado_proyecto
    # Impide la creacion si no hay estado actual
    if id.nil? && !proyecto.estado_actual
      errors.add("Estado", _("El proyecto no tiene asignado ningún estado todavía.")+ _("No se pueden modificar los periodos.") )
    # O el borrado y modificacion si el proyecto esta cerrado
    elsif proyecto.estado_actual && proyecto.estado_actual.definicion_estado && proyecto.estado_actual.definicion_estado.cerrado
      errors.add("Estado", _("El proyecto se encuentra en estado '%{estado}'.")%{:estado => proyecto.estado_actual.definicion_estado.nombre} + _("No se pueden modificar los periodos.") )
    end
    return errors.empty? 
  end

end
